@@ -1,12 +1,12 @@
 package Pcore::API::Docker::Hub;
 
 use Pcore -const, -class, -res, -export;
-use Pcore::Util::Scalar qw[is_plain_coderef];
+use Pcore::Lib::Scalar qw[is_plain_coderef];
 
 our $EXPORT = { DOCKERHUB_SOURCE_TYPE => [qw[$DOCKERHUB_SOURCE_TYPE_TAG $DOCKERHUB_SOURCE_TYPE_BRANCH]] };
 
 has username => ( required => 1 );
-has password => ( required => 1 );
+has token    => ( required => 1 );
 
 has _login_token => ( init_arg => undef );
 has _reg_queue   => ( init_arg => undef );    # HashRef [ArrayRef]
@@ -38,7 +38,7 @@ const our $BUILD_STATUS_TEXT => {
 sub BUILDARGS ( $self, $args = undef ) {
     $args->{username} ||= $ENV->user_cfg->{DOCKERHUB}->{username};
 
-    $args->{password} ||= $ENV->user_cfg->{DOCKERHUB}->{password};
+    $args->{token} ||= $ENV->user_cfg->{DOCKERHUB}->{token};
 
     return $args;
 }
@@ -61,7 +61,7 @@ sub _login ( $self, $cb ) {
         $endpoint,
         undef,
         {   username => $self->{username},
-            password => $self->{password},
+            password => $self->{token},
         },
         sub ($res) {
             if ( !$res ) {
@@ -161,11 +161,12 @@ sub get_user_orgs ( $self, $cb = undef ) {
 }
 
 # CREATE REPO / AUTOMATED BUILD
-sub create_repo ( $self, $repo_id, $desc, @args ) {
+sub create_repo ( $self, $repo_id, @args ) {
     my $cb = is_plain_coderef $args[-1] ? pop @args : undef;
 
     my %args = (
         private   => 0,
+        desc      => $EMPTY,
         full_desc => $EMPTY,
         @args
     );
@@ -179,14 +180,15 @@ sub create_repo ( $self, $repo_id, $desc, @args ) {
         {   namespace        => $namespace,
             name             => $name,
             is_private       => $args{private},
-            description      => $desc,
+            description      => $args{desc},
             full_description => $args{full_desc},
         },
         $cb
     );
 }
 
-sub create_autobuild ( $self, $repo_id, $scm_provider, $scm_repo_id, $desc, @args ) {
+# TODO not work
+sub create_autobuild ( $self, $repo_id, $git_repo_id, $desc, @args ) {
     my $cb = is_plain_coderef $args[-1] ? pop @args : undef;
 
     my %args = (
@@ -231,8 +233,8 @@ sub create_autobuild ( $self, $repo_id, $scm_provider, $scm_repo_id, $desc, @arg
             is_private          => $args{private} ? \1 : \0,
             active              => $args{active} ? \1 : \0,
             dockerhub_repo_name => $repo_id,
-            provider            => $scm_provider,
-            vcs_repo_name       => $scm_repo_id,
+            provider            => 'git',
+            vcs_repo_name       => $git_repo_id,
             description         => $desc,
             build_tags          => $build_tags,
         },
@@ -283,14 +285,6 @@ sub remove_repo ( $self, $repo_id, $cb = undef ) {
     return $self->_req( 'DELETE', "/repositories/$repo_id/", 1, undef, $cb );
 }
 
-sub set_desc ( $self, $repo_id, $desc, $cb = undef ) {
-    return $self->_req( 'PATCH', "/repositories/$repo_id/", 1, { description => $desc }, $cb );
-}
-
-sub set_full_desc ( $self, $repo_id, $desc, $cb = undef ) {
-    return $self->_req( 'PATCH', "/repositories/$repo_id/", 1, { full_description => $desc }, $cb );
-}
-
 # REPO TAGS
 # TODO gel all pages
 sub get_tags ( $self, $repo_id, $cb = undef ) {
@@ -403,48 +397,6 @@ sub get_autobuild_settings ( $self, $repo_id, $cb = undef ) {
     return $self->_req( 'GET', "/repositories/$repo_id/autobuild/", 1, undef, $cb );
 }
 
-sub unlink_tag ( $self, $repo_id, $tag_name, $cb = undef ) {
-    my ( $delete_autobuild_tag_status, $delete_tag_status );
-
-    my $cv = P->cv->begin( sub ($cv) {
-        my $res = res [ 200, "autobuild: $delete_autobuild_tag_status->{reason}, tag: $delete_tag_status->{reason}" ];
-
-        $cv->( $cb ? $cb->($res) : $res );
-
-        return;
-    } );
-
-    $cv->begin;
-    $self->delete_autobuild_tag_by_name(
-        $repo_id,
-        $tag_name,
-        sub ($res) {
-            $delete_autobuild_tag_status = $res;
-
-            $cv->end;
-
-            return;
-        }
-    );
-
-    $cv->begin;
-    $self->delete_tag(
-        $repo_id,
-        $tag_name,
-        sub ($res) {
-            $delete_tag_status = $res;
-
-            $cv->end;
-
-            return;
-        }
-    );
-
-    $cv->end;
-
-    return defined wantarray ? $cv->recv : ();
-}
-
 # AUTOBUILD TAGS
 sub get_autobuild_tags ( $self, $repo_id, $cb = undef ) {
     return $self->_req(
@@ -586,10 +538,9 @@ sub trigger_autobuild_by_tag_name ( $self, $repo_id, $autobuild_tag_name, $cb =
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
-## |    3 | 83, 189, 317, 327,   | Subroutines::ProhibitManyArgs - Too many arguments                                                             |
-## |      | 343, 369, 373, 406,  |                                                                                                                |
-## |      | 470, 489, 493, 531,  |                                                                                                                |
-## |      | 544                  |                                                                                                                |
+## |    3 | 83, 191, 311, 321,   | Subroutines::ProhibitManyArgs - Too many arguments                                                             |
+## |      | 337, 363, 367, 422,  |                                                                                                                |
+## |      | 441, 445, 483, 496   |                                                                                                                |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
 ## |    1 | 167                  | CodeLayout::RequireTrailingCommas - List declaration without trailing comma                                    |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
